@@ -51,6 +51,7 @@
 
 #include "torrent/chunk_manager.h"
 #include "torrent/connection_manager.h"
+#include "torrent/peer/client_list.h"
 
 #include "manager.h"
 #include "resource_manager.h"
@@ -67,6 +68,7 @@ Manager::Manager() :
   m_resourceManager(new ResourceManager),
 
   m_chunkManager(new ChunkManager),
+  m_clientList(new ClientList),
   m_connectionManager(new ConnectionManager),
 
   m_poll(NULL),
@@ -81,6 +83,7 @@ Manager::Manager() :
   priority_queue_insert(&taskScheduler, &m_taskTick, cachedTime.round_seconds());
 
   m_handshakeManager->slot_download_id(rak::make_mem_fun(m_downloadManager, &DownloadManager::find_main));
+  m_handshakeManager->slot_download_id_obfuscated(rak::make_mem_fun(m_downloadManager, &DownloadManager::find_main_obfuscated));
   m_connectionManager->listen()->slot_incoming(rak::make_mem_fun(m_handshakeManager, &HandshakeManager::add_incoming));
 }
 
@@ -99,6 +102,8 @@ Manager::~Manager() {
   delete m_connectionManager;
   delete m_chunkManager;
 
+  delete m_clientList;
+
   delete m_uploadThrottle;
   delete m_downloadThrottle;
 }
@@ -109,11 +114,6 @@ Manager::initialize_download(DownloadWrapper* d) {
   d->main()->slot_start_handshake(rak::make_mem_fun(m_handshakeManager, &HandshakeManager::add_outgoing));
   d->main()->slot_stop_handshakes(rak::make_mem_fun(m_handshakeManager, &HandshakeManager::erase_download));
 
-  d->hash_checker()->set_queue(m_hashQueue);
-
-  d->main()->content()->entry_list()->slot_insert_filemeta(rak::make_mem_fun(m_fileManager, &FileManager::insert));
-  d->main()->content()->entry_list()->slot_erase_filemeta(rak::make_mem_fun(m_fileManager, &FileManager::erase));
-
   m_downloadManager->insert(d);
   m_resourceManager->insert(d->main(), 1);
   m_chunkManager->insert(d->main()->chunk_list());
