@@ -63,10 +63,10 @@ Listen::open(uint16_t first, uint16_t last, const rak::socket_address* bindAddre
     throw input_error("Listening socket must be bound to an inet or inet6 address.");
 
   if (!get_fd().open_stream() || !get_fd().set_nonblock())
-    throw local_error("Could not allocate socket for listening.");
+    throw resource_error("Could not allocate socket for listening.");
 
   if (!get_fd().set_reuse_address(true))
-    throw local_error("Could not set listening port to reuse address.");
+    throw resource_error("Could not set listening port to reuse address.");
 
   rak::socket_address sa;
   sa.copy(*bindAddress, bindAddress->length());
@@ -126,7 +126,7 @@ Listen::event_write() {
 
 void
 Listen::event_error() {
-  throw local_error("Listener port received an error event.");
+  throw resource_error("Listener port received an error event.");
 }
 
 }
