@@ -42,7 +42,7 @@
 
 #include "net/throttle_node.h"
 #include "torrent/bitfield.h"
-#include "torrent/piece.h"
+#include "torrent/data/piece.h"
 #include "torrent/rate.h"
 
 namespace torrent {
@@ -76,6 +76,7 @@ public:
 
   piece_list_type*    upload_queue()                { return &m_uploadQueue; }
   index_list_type*    have_queue()                  { return &m_haveQueue; }
+  piece_list_type*    cancel_queue()                { return &m_cancelQueue; }
 
   Rate*               peer_rate()                   { return &m_peerRate; }
 
@@ -94,6 +95,7 @@ private:
 
   piece_list_type     m_uploadQueue;
   index_list_type     m_haveQueue;
+  piece_list_type     m_cancelQueue;
 
   Rate                m_peerRate;
 
