@@ -40,7 +40,8 @@
 #include <rak/error_number.h>
 
 #include "torrent/exceptions.h"
-#include "torrent/block.h"
+#include "torrent/data/block.h"
+#include "torrent/chunk_manager.h"
 #include "data/chunk_iterator.h"
 #include "data/chunk_list.h"
 #include "download/choke_manager.h"
@@ -50,7 +51,7 @@
 #include "download/download_main.h"
 #include "net/socket_base.h"
 #include "torrent/connection_manager.h"
-#include "torrent/peer_info.h"
+#include "torrent/peer/peer_info.h"
 
 #include "peer_connection_base.h"
 
@@ -69,27 +70,36 @@ PeerConnectionBase::PeerConnectionBase() :
   m_downStall(0),
 
   m_sendChoked(false),
-  m_sendInterested(false) {
+  m_sendInterested(false),
+
+  m_encryptBuffer(NULL) {
 }
 
 PeerConnectionBase::~PeerConnectionBase() {
   delete m_up;
   delete m_down;
+
+  delete m_encryptBuffer;
 }
 
 void
-PeerConnectionBase::initialize(DownloadMain* download, PeerInfo* peerInfo, SocketFd fd, Bitfield* bitfield) {
+PeerConnectionBase::initialize(DownloadMain* download, PeerInfo* peerInfo, SocketFd fd, Bitfield* bitfield, EncryptionInfo* encryptionInfo) {
   if (get_fd().is_valid())
     throw internal_error("Tried to re-set PeerConnection.");
 
-  if (!peerInfo->is_valid() || !fd.is_valid())
+  if (!fd.is_valid())
     throw internal_error("PeerConnectionBase::set(...) received bad input.");
 
+  if (encryptionInfo->is_encrypted() != encryptionInfo->decrypt_valid())
+    throw internal_error("Encryption and decryption inconsistent.");
+
   set_fd(fd);
 
   m_peerInfo = peerInfo;
   m_download = download;
 
+  m_encryption = *encryptionInfo;
+
   m_peerChunks.set_peer_info(m_peerInfo);
   m_peerChunks.bitfield()->swap(*bitfield);
 
@@ -167,12 +177,29 @@ PeerConnectionBase::load_up_chunk() {
   if (!m_upChunk.is_valid())
     throw storage_error("File chunk read error: " + std::string(m_upChunk.error_number().c_str()));
 
-  // Make sure we preload the next step once we get past the length
-  // here. This is just some testing, don't include this with the
-  // release. (Yet)
-//   if (m_peerChunks.upload_throttle()->rate()->rate() >= 10 << 10)
-//     m_upChunk.chunk()->preload(m_upPiece.offset(), m_upChunk.chunk()->size());
-//   m_upChunk.chunk()->preload(m_upPiece.offset(), std::min(128u << 10, m_peerChunks.upload_throttle()->rate()->rate() * 10));
+  if (is_encrypted() && m_encryptBuffer == NULL) {
+    m_encryptBuffer = new EncryptBuffer();
+    m_encryptBuffer->reset();
+  }
+
+  // Also check if we've already preloaded in the recent past, even
+  // past unmaps.
+  ChunkManager* cm = manager->chunk_manager();
+  uint32_t preloadSize = m_upChunk.chunk()->chunk_size() - m_upPiece.offset();
+
+  if (cm->preload_type() == 0 ||
+      m_upChunk.object()->time_preloaded() >= cachedTime - rak::timer::from_seconds(60) ||
+
+      preloadSize < cm->preload_min_size() ||
+      m_peerChunks.upload_throttle()->rate()->rate() < cm->preload_required_rate() * ((preloadSize + (2 << 20) - 1) / (2 << 20))) {
+    cm->inc_stats_not_preloaded();
+    return;
+  }
+
+  cm->inc_stats_preloaded();
+
+  m_upChunk.object()->set_time_preloaded(cachedTime);
+  m_upChunk.chunk()->preload(m_upPiece.offset(), m_upChunk.chunk()->chunk_size(), cm->preload_type() == 1);
 }
 
 void
@@ -205,6 +232,22 @@ PeerConnectionBase::receive_choke(bool v) {
   m_timeLastChoked = cachedTime;
 }
 
+void
+PeerConnectionBase::cancel_transfer(BlockTransfer* transfer) {
+  if (!get_fd().is_valid())
+    throw internal_error("PeerConnectionBase::cancel_transfer(...) !get_fd().is_valid().");
+
+  // We don't send cancel messages if the transfer has already
+  // started.
+  if (transfer == m_downloadQueue.transfer())
+    return;
+
+  write_insert_poll_safe();
+
+  m_peerChunks.cancel_queue()->push_back(transfer->piece());
+//   m_downloadQueue.cancel_transfer(transfer);
+}
+
 void
 PeerConnectionBase::receive_throttle_down_activate() {
   manager->poll()->insert_read(this);
@@ -229,7 +272,7 @@ PeerConnectionBase::down_chunk_start(const Piece& piece) {
     return false;
   }
 
-  if (!m_download->content()->is_valid_piece(piece))
+  if (!m_download->file_list()->is_valid_piece(piece))
     throw internal_error("Incoming pieces list contains a bad piece.");
   
   if (!m_downChunk.is_valid() || piece.index() != m_downChunk.index()) {
@@ -295,6 +338,9 @@ PeerConnectionBase::down_chunk() {
     data = itr.data();
     data.second = read_stream_throws(data.first, data.second);
 
+    if (is_encrypted())
+      m_encryption.decrypt(data.first, data.second);
+
     bytesTransfered += data.second;
 
   } while (itr.used(data.second));
@@ -309,7 +355,7 @@ PeerConnectionBase::down_chunk() {
 
 bool
 PeerConnectionBase::down_chunk_from_buffer() {
-  m_down->buffer()->move_position(down_chunk_process(m_down->buffer()->position(), m_down->buffer()->remaining()));
+  m_down->buffer()->consume(down_chunk_process(m_down->buffer()->position(), m_down->buffer()->remaining()));
 
   if (!m_downloadQueue.transfer()->is_finished() && m_down->buffer()->remaining() != 0)
     throw internal_error("PeerConnectionBase::down_chunk_from_buffer() !transfer->is_finished() && m_down->buffer()->remaining() != 0.");
@@ -324,6 +370,9 @@ bool
 PeerConnectionBase::down_chunk_skip() {
   uint32_t length = read_stream_throws(m_nullBuffer, m_downloadQueue.transfer()->piece().length() - m_downloadQueue.transfer()->position());
 
+  if (is_encrypted())
+    m_encryption.decrypt(m_nullBuffer, length);
+
   if (down_chunk_skip_process(m_nullBuffer, length) != length)
     throw internal_error("PeerConnectionBase::down_chunk_skip() down_chunk_skip_process(m_nullBuffer, length) != length.");
 
@@ -332,7 +381,7 @@ PeerConnectionBase::down_chunk_skip() {
 
 bool
 PeerConnectionBase::down_chunk_skip_from_buffer() {
-  m_down->buffer()->move_position(down_chunk_skip_process(m_down->buffer()->position(), m_down->buffer()->remaining()));
+  m_down->buffer()->consume(down_chunk_skip_process(m_down->buffer()->position(), m_down->buffer()->remaining()));
   
   return m_downloadQueue.transfer()->is_finished();
 }
@@ -371,8 +420,11 @@ PeerConnectionBase::down_chunk_skip_process(const void* buffer, uint32_t length)
   // the block to simplify the rest of the function.
   length = std::min(length, transfer->piece().length() - transfer->position());
 
+  // Hmm, this might result in more bytes than nessesary being
+  // counted.
   m_download->download_throttle()->node_used(m_peerChunks.download_throttle(), length);
   m_download->info()->down_rate()->insert(length);
+  m_download->info()->skip_rate()->insert(length);
 
   if (!transfer->is_valid()) {
     transfer->adjust_position(length);
@@ -417,6 +469,35 @@ PeerConnectionBase::down_chunk_skip_process(const void* buffer, uint32_t length)
   return length;
 }
 
+inline uint32_t
+PeerConnectionBase::up_chunk_encrypt(uint32_t quota) {
+  if (m_encryptBuffer == NULL)
+    throw internal_error("PeerConnectionBase::up_chunk: m_encryptBuffer is NULL.");
+
+  if (quota <= m_encryptBuffer->remaining())
+    return quota;
+
+  // Also, consider checking here if the number of bytes remaining in
+  // the buffer is small enought that the cost of moving them would
+  // outweigh the extra context switches, etc.
+
+  if (m_encryptBuffer->remaining() == 0) {
+    // This handles reset also for new chunk transfers.
+    m_encryptBuffer->reset();
+
+    quota = std::min<uint32_t>(quota, m_encryptBuffer->reserved());
+
+  } else {
+    quota = std::min<uint32_t>(quota - m_encryptBuffer->remaining(), m_encryptBuffer->reserved_left());
+  }
+
+  m_upChunk.chunk()->to_buffer(m_encryptBuffer->end(), m_upPiece.offset() + m_encryptBuffer->remaining(), quota);
+  m_encryption.encrypt(m_encryptBuffer->end(), quota);
+  m_encryptBuffer->move_end(quota);
+
+  return m_encryptBuffer->remaining();
+}
+
 bool
 PeerConnectionBase::up_chunk() {
   if (!m_download->upload_throttle()->is_throttled(m_peerChunks.upload_throttle()))
@@ -435,16 +516,27 @@ PeerConnectionBase::up_chunk() {
 
   uint32_t bytesTransfered = 0;
 
-  Chunk::data_type data;
-  ChunkIterator itr(m_upChunk.chunk(), m_upPiece.offset(), m_upPiece.offset() + std::min(quota, m_upPiece.length()));
+  if (is_encrypted()) {
+    // Prepare as many bytes as quota specifies, up to end of piece or
+    // buffer. Only bytes beyond remaining() are new and will be
+    // encrypted.
+    quota = up_chunk_encrypt(std::min(quota, m_upPiece.length()));
 
-  do {
-    data = itr.data();
-    data.second = write_stream_throws(data.first, data.second);
+    bytesTransfered = write_stream_throws(m_encryptBuffer->position(), quota);
+    m_encryptBuffer->consume(bytesTransfered);
 
-    bytesTransfered += data.second;
+  } else {
+    Chunk::data_type data;
+    ChunkIterator itr(m_upChunk.chunk(), m_upPiece.offset(), m_upPiece.offset() + std::min(quota, m_upPiece.length()));
 
-  } while (itr.used(data.second));
+    do {
+      data = itr.data();
+      data.second = write_stream_throws(data.first, data.second);
+
+      bytesTransfered += data.second;
+
+    } while (itr.used(data.second));
+  }
 
   m_download->upload_throttle()->node_used(m_peerChunks.upload_throttle(), bytesTransfered);
   m_download->info()->up_rate()->insert(bytesTransfered);
@@ -488,24 +580,14 @@ PeerConnectionBase::read_cancel_piece(const Piece& p) {
     m_peerChunks.upload_queue()->erase(itr);
 }  
 
-void
-PeerConnectionBase::read_buffer_move_unused() {
-  uint32_t remaining = m_down->buffer()->remaining();
-  
-  std::memmove(m_down->buffer()->begin(), m_down->buffer()->position(), remaining);
-  
-  m_down->buffer()->reset_position();
-  m_down->buffer()->set_end(remaining);
-}
-
 void
 PeerConnectionBase::write_prepare_piece() {
   m_upPiece = m_peerChunks.upload_queue()->front();
   m_peerChunks.upload_queue()->pop_front();
 
   // Move these checks somewhere else?
-  if (!m_download->content()->is_valid_piece(m_upPiece) ||
-      !m_download->content()->has_chunk(m_upPiece.index())) {
+  if (!m_download->file_list()->is_valid_piece(m_upPiece) ||
+      !m_download->file_list()->bitfield()->get(m_upPiece.index())) {
     std::stringstream s;
 
     s << "Peer requested a piece with invalid index or length/offset: "
@@ -564,7 +646,7 @@ PeerConnectionBase::try_request_pieces() {
     if (p == NULL)
       break;
 
-    if (!m_download->content()->is_valid_piece(*p) || !m_peerChunks.bitfield()->get(p->index()))
+    if (!m_download->file_list()->is_valid_piece(*p) || !m_peerChunks.bitfield()->get(p->index()))
       throw internal_error("PeerConnectionBase::try_request_pieces() tried to use an invalid piece.");
 
     m_up->write_request(*p);
