@@ -41,6 +41,7 @@
 #include "net/socket_stream.h"
 #include "torrent/poll.h"
 
+#include "encryption_info.h"
 #include "peer_chunks.h"
 #include "protocol_base.h"
 #include "request_list.h"
@@ -65,13 +66,20 @@ public:
   typedef ProtocolBase           ProtocolRead;
   typedef ProtocolBase           ProtocolWrite;
 
+#if USE_EXTRA_DEBUG == 666
+  // For testing, use a really small buffer.
+  typedef ProtocolBuffer<256>  EncryptBuffer;
+#else
+  typedef ProtocolBuffer<16384>  EncryptBuffer;
+#endif
+
   // Find an optimal number for this.
   static const uint32_t read_size = 64;
 
   PeerConnectionBase();
   virtual ~PeerConnectionBase();
   
-  void                initialize(DownloadMain* download, PeerInfo* p, SocketFd fd, Bitfield* bitfield);
+  void                initialize(DownloadMain* download, PeerInfo* p, SocketFd fd, Bitfield* bitfield, EncryptionInfo* encryptionInfo);
   void                cleanup();
 
   bool                is_up_choked()                { return m_up->choked(); }
@@ -83,6 +91,9 @@ public:
 
   bool                is_seeder() const             { return m_peerChunks.is_seeder(); }
 
+  bool                is_encrypted() const          { return m_encryption.is_encrypted(); }
+  bool                is_obfuscated() const         { return m_encryption.is_obfuscated(); }
+
   PeerInfo*           peer_info()                   { return m_peerInfo; }
   const PeerInfo*     c_peer_info() const           { return m_peerInfo; }
   PeerChunks*         peer_chunks()                 { return &m_peerChunks; }
@@ -110,6 +121,8 @@ public:
 
   void                push_unread(const void* data, uint32_t size);
 
+  void                cancel_transfer(BlockTransfer* transfer);
+
 protected:
   inline bool         read_remaining();
   inline bool         write_remaining();
@@ -122,8 +135,6 @@ protected:
   void                read_request_piece(const Piece& p);
   void                read_cancel_piece(const Piece& p);
 
-  void                read_buffer_move_unused();
-
   void                write_prepare_piece();
 
   bool                down_chunk_start(const Piece& p);
@@ -138,6 +149,7 @@ protected:
   uint32_t            down_chunk_skip_process(const void* buffer, uint32_t length);
 
   bool                up_chunk();
+  inline uint32_t     up_chunk_encrypt(uint32_t quota);
 
   void                down_chunk_release();
   void                up_chunk_release();
@@ -172,6 +184,9 @@ protected:
 
   rak::timer          m_timeLastChoked;
   rak::timer          m_timeLastRead;
+
+  EncryptBuffer*      m_encryptBuffer;
+  EncryptionInfo      m_encryption;
 };
 
 inline void
@@ -180,20 +195,6 @@ PeerConnectionBase::push_unread(const void* data, uint32_t size) {
   m_down->buffer()->move_end(size);
 }
 
-inline bool
-PeerConnectionBase::read_remaining() {
-  m_down->buffer()->move_position(read_stream_throws(m_down->buffer()->position(), m_down->buffer()->remaining()));
-
-  return !m_down->buffer()->remaining();
-}
-
-inline bool
-PeerConnectionBase::write_remaining() {
-  m_up->buffer()->move_position(write_stream_throws(m_up->buffer()->position(), m_up->buffer()->remaining()));
-
-  return !m_up->buffer()->remaining();
-}
-
 inline void
 PeerConnectionBase::read_insert_poll_safe() {
   if (m_down->get_state() != ProtocolRead::IDLE)
