@@ -39,7 +39,6 @@
 #include <cstring>
 #include <sstream>
 
-#include "data/content.h"
 #include "data/chunk_list_node.h"
 #include "download/chunk_selector.h"
 #include "download/chunk_statistics.h"
@@ -60,7 +59,7 @@ PeerConnectionLeech::~PeerConnectionLeech() {
 void
 PeerConnectionLeech::initialize_custom() {
 //   if (m_download->content()->chunks_completed() != 0) {
-//     m_up->write_bitfield(m_download->content()->bitfield()->size_bytes());
+//     m_up->write_bitfield(m_download->file_list()->bitfield()->size_bytes());
 
 //     m_up->buffer()->prepare_end();
 //     m_up->set_position(0);
@@ -108,7 +107,11 @@ PeerConnectionLeech::receive_keepalive() {
       m_up->can_write_keepalive()) {
 
     write_insert_poll_safe();
+
+    ProtocolBuffer<512>::iterator old_end = m_up->buffer()->end();
     m_up->write_keepalive();
+    if (is_encrypted())
+      m_encryption.encrypt(old_end, m_up->buffer()->end() - old_end);
   }
 
   m_tryRequest = true;
@@ -153,7 +156,7 @@ PeerConnectionLeech::read_message() {
     return false;
 
   } else if (length > (1 << 20)) {
-    throw network_error("PeerConnectionLeech::read_message() got an invalid message length.");
+    throw communication_error("PeerConnectionLeech::read_message() got an invalid message length.");
   }
     
   // We do not verify the message length of those with static
@@ -257,7 +260,7 @@ PeerConnectionLeech::read_message() {
     return true;
 
   default:
-    throw network_error("Received unsupported message type.");
+    throw communication_error("Received unsupported message type.");
   }
 
   // We were unsuccessfull in reading the message, need more data.
@@ -289,24 +292,28 @@ PeerConnectionLeech::event_read() {
 
       switch (m_down->get_state()) {
       case ProtocolRead::IDLE:
-        if (m_down->buffer()->size_end() == read_size)
-          throw internal_error("PeerConnectionLeech::event_read() m_down->buffer()->size_end() == read_size.");
+        if (m_down->buffer()->size_end() < read_size) {
+          unsigned int length = read_stream_throws(m_down->buffer()->end(), read_size - m_down->buffer()->size_end());
+
+          if (is_encrypted())
+            m_encryption.decrypt(m_down->buffer()->end(), length);
+
+          m_down->buffer()->move_end(length);
+        }
 
-        m_down->buffer()->move_end(read_stream_throws(m_down->buffer()->end(), read_size - m_down->buffer()->size_end()));
-        
         while (read_message());
         
         if (m_down->buffer()->size_end() == read_size) {
-          read_buffer_move_unused();
+          m_down->buffer()->move_unused();
           break;
         } else {
-          read_buffer_move_unused();
+          m_down->buffer()->move_unused();
           return;
         }
 
       case ProtocolRead::READ_PIECE:
         if (!download_queue()->is_downloading())
-          throw internal_error("ProtocolRead::READ_PIECE state but RequestList is not downloading");
+          throw internal_error("ProtocolRead::READ_PIECE state but RequestList is not downloading.");
 
         if (!m_downloadQueue.transfer()->is_valid() || !m_downloadQueue.transfer()->is_leader()) {
           m_down->set_state(ProtocolRead::READ_SKIP_PIECE);
@@ -365,14 +372,14 @@ PeerConnectionLeech::event_read() {
     std::stringstream s;
     s << "Connection read fd(" << get_fd().get_fd() << ',' << m_down->get_state() << ',' << m_down->last_command() << ") \"" << e.what() << '"';
 
-    e.set(s.str());
-
-    throw;
+    throw internal_error(s.str());
   }
 }
 
 inline void
 PeerConnectionLeech::fill_write_buffer() {
+  ProtocolBuffer<512>::iterator old_end = m_up->buffer()->end();
+
   // No need to use delayed choke as we are a leecher.
   if (m_sendChoked && m_up->can_write_choke()) {
     m_sendChoked = false;
@@ -383,6 +390,14 @@ PeerConnectionLeech::fill_write_buffer() {
       up_chunk_release();
       m_peerChunks.upload_queue()->clear();
 
+      if (m_encryptBuffer != NULL) {
+        if (m_encryptBuffer->remaining())
+          throw internal_error("Deleting encryptBuffer with encrypted data remaining.");
+
+        delete m_encryptBuffer;
+        m_encryptBuffer = NULL;
+      }
+
     } else {
       m_download->upload_throttle()->insert(m_peerChunks.upload_throttle());
     }
@@ -412,10 +427,18 @@ PeerConnectionLeech::fill_write_buffer() {
     m_peerChunks.have_queue()->pop_front();
   }
 
+  while (!m_peerChunks.cancel_queue()->empty() && m_up->can_write_cancel()) {
+    m_up->write_cancel(m_peerChunks.cancel_queue()->front());
+    m_peerChunks.cancel_queue()->pop_front();
+  }
+
   if (!m_up->choked() &&
       !m_peerChunks.upload_queue()->empty() &&
       m_up->can_write_piece())
     write_prepare_piece();
+
+  if (is_encrypted())
+    m_encryption.encrypt(old_end, m_up->buffer()->end() - old_end);
 }
 
 void
@@ -427,26 +450,17 @@ PeerConnectionLeech::event_write() {
       switch (m_up->get_state()) {
       case ProtocolWrite::IDLE:
 
-        // We might have buffered keepalive message or similar, but
-        // 'end' should remain at the start of the buffer.
-        if (m_up->buffer()->size_end() != 0)
-          throw internal_error("PeerConnectionLeech::event_write() ProtocolWrite::IDLE in a wrong state.");
-
-        // Fill up buffer.
         fill_write_buffer();
 
-        if (m_up->buffer()->size_position() == 0) {
+        if (m_up->buffer()->remaining() == 0) {
           manager->poll()->remove_write(this);
           return;
         }
 
         m_up->set_state(ProtocolWrite::MSG);
-        m_up->buffer()->prepare_end();
 
       case ProtocolWrite::MSG:
-        m_up->buffer()->move_position(write_stream_throws(m_up->buffer()->position(), m_up->buffer()->remaining()));
-
-        if (m_up->buffer()->remaining())
+        if (!m_up->buffer()->consume(write_stream_throws(m_up->buffer()->position(), m_up->buffer()->remaining())))
           return;
 
         m_up->buffer()->reset();
@@ -460,6 +474,7 @@ PeerConnectionLeech::event_write() {
 
         // We're uploading a piece.
         load_up_chunk();
+
         m_up->set_state(ProtocolWrite::WRITE_PIECE);
 
       case ProtocolWrite::WRITE_PIECE:
@@ -495,29 +510,27 @@ PeerConnectionLeech::event_write() {
     std::stringstream s;
     s << "Connection write fd(" << get_fd().get_fd() << ',' << m_up->get_state() << ',' << m_up->last_command() << ") \"" << e.what() << '"';
 
-    e.set(s.str());
-
-    throw;
+    throw internal_error(s.str());
   }
 }
 
 void
 PeerConnectionLeech::read_have_chunk(uint32_t index) {
   if (index >= m_peerChunks.bitfield()->size_bits())
-    throw network_error("Peer sent HAVE message with out-of-range index.");
+    throw communication_error("Peer sent HAVE message with out-of-range index.");
 
   if (m_peerChunks.bitfield()->get(index))
     return;
 
-  m_download->chunk_statistics()->received_have_chunk(&m_peerChunks, index, m_download->content()->chunk_size());
+  m_download->chunk_statistics()->received_have_chunk(&m_peerChunks, index, m_download->file_list()->chunk_size());
 
   if (m_peerChunks.bitfield()->is_all_set())
-    if (m_download->content()->is_done())
+    if (m_download->file_list()->is_done())
       throw close_connection();
     else
       set_remote_not_interested();
 
-  if (m_download->content()->is_done())
+  if (m_download->file_list()->is_done())
     return;
 
   if (is_up_interested()) {
