@@ -39,7 +39,6 @@
 #include <cstring>
 #include <sstream>
 
-#include "data/content.h"
 #include "download/chunk_statistics.h"
 #include "download/download_info.h"
 #include "download/download_main.h"
@@ -58,7 +57,7 @@ PeerConnectionSeed::~PeerConnectionSeed() {
 void
 PeerConnectionSeed::initialize_custom() {
 //   if (m_download->content()->chunks_completed() != 0) {
-//     m_up->write_bitfield(m_download->content()->bitfield()->size_bytes());
+//     m_up->write_bitfield(m_download->file_list()->bitfield()->size_bytes());
 
 //     m_up->buffer()->prepare_end();
 //     m_up->set_position(0);
@@ -120,7 +119,7 @@ PeerConnectionSeed::read_message() {
     return false;
 
   } else if (length > (1 << 20)) {
-    throw network_error("PeerConnectionSeed::read_message() got an invalid message length.");
+    throw communication_error("PeerConnectionSeed::read_message() got an invalid message length.");
   }
     
   // We do not verify the message length of those with static
@@ -174,7 +173,7 @@ PeerConnectionSeed::read_message() {
     return true;
 
   case ProtocolBase::PIECE:
-    throw network_error("Received a piece but the connection is strictly for seeding.");
+    throw communication_error("Received a piece but the connection is strictly for seeding.");
 
   case ProtocolBase::CANCEL:
     if (!m_down->can_read_cancel_body())
@@ -184,7 +183,7 @@ PeerConnectionSeed::read_message() {
     return true;
 
   default:
-    throw network_error("Received unsupported message type.");
+    throw communication_error("Received unsupported message type.");
   }
 
   // We were unsuccessfull in reading the message, need more data.
@@ -222,10 +221,9 @@ PeerConnectionSeed::event_read() {
       while (read_message());
         
       if (m_down->buffer()->size_end() == read_size) {
-        read_buffer_move_unused();
-
+        m_down->buffer()->move_unused();
       } else {
-        read_buffer_move_unused();
+        m_down->buffer()->move_unused();
         return;
       }
 
@@ -254,9 +252,7 @@ PeerConnectionSeed::event_read() {
     std::stringstream s;
     s << "Connection read fd(" << get_fd().get_fd() << ',' << m_down->get_state() << ',' << m_down->last_command() << ") \"" << e.what() << '"';
 
-    e.set(s.str());
-
-    throw;
+    throw internal_error(s.str());
   }
 }
 
@@ -292,26 +288,18 @@ PeerConnectionSeed::event_write() {
       switch (m_up->get_state()) {
       case ProtocolWrite::IDLE:
 
-        // We might have buffered keepalive message or similar, but
-        // 'end' should remain at the start of the buffer.
-        if (m_up->buffer()->size_end() != 0)
-          throw internal_error("PeerConnectionSeed::event_write() ProtocolWrite::IDLE in a wrong state.");
-
         // Fill up buffer.
         fill_write_buffer();
 
-        if (m_up->buffer()->size_position() == 0) {
+        if (m_up->buffer()->size_end() == 0) {
           manager->poll()->remove_write(this);
           return;
         }
 
         m_up->set_state(ProtocolWrite::MSG);
-        m_up->buffer()->prepare_end();
 
       case ProtocolWrite::MSG:
-        m_up->buffer()->move_position(write_stream_throws(m_up->buffer()->position(), m_up->buffer()->remaining()));
-
-        if (m_up->buffer()->remaining())
+        if (!m_up->buffer()->consume(write_stream_throws(m_up->buffer()->position(), m_up->buffer()->remaining())))
           return;
 
         m_up->buffer()->reset();
@@ -360,21 +348,19 @@ PeerConnectionSeed::event_write() {
     std::stringstream s;
     s << "Connection write fd(" << get_fd().get_fd() << ',' << m_up->get_state() << ',' << m_up->last_command() << ") \"" << e.what() << '"';
 
-    e.set(s.str());
-
-    throw;
+    throw internal_error(s.str());
   }
 }
 
 void
 PeerConnectionSeed::read_have_chunk(uint32_t index) {
   if (index >= m_peerChunks.bitfield()->size_bits())
-    throw network_error("Peer sent HAVE message with out-of-range index.");
+    throw communication_error("Peer sent HAVE message with out-of-range index.");
 
   if (m_peerChunks.bitfield()->get(index))
     return;
 
-  m_download->chunk_statistics()->received_have_chunk(&m_peerChunks, index, m_download->content()->chunk_size());
+  m_download->chunk_statistics()->received_have_chunk(&m_peerChunks, index, m_download->file_list()->chunk_size());
   //m_download->chunk_selector()->received_have_chunk(&m_peerChunks, index);
 
   if (m_peerChunks.bitfield()->is_all_set())
