@@ -40,8 +40,8 @@
 #include <inttypes.h>
 #include <rak/functional.h>
 
-#include "torrent/block.h"
-#include "torrent/block_list.h"
+#include "torrent/data/block.h"
+#include "torrent/data/block_list.h"
 #include "torrent/exceptions.h"
 #include "download/delegator.h"
 
@@ -140,7 +140,7 @@ RequestList::downloading(const Piece& piece) {
   // the unmodified BlockTransfer.
   if (piece.length() != m_transfer->piece().length()) {
     if (piece.length() != 0)
-      throw network_error("Peer sent a piece with wrong, non-zero, length.");
+      throw communication_error("Peer sent a piece with wrong, non-zero, length.");
 
     Block::release(m_transfer);
     goto downloading_error;
@@ -158,7 +158,7 @@ RequestList::downloading(const Piece& piece) {
   // Create a dummy BlockTransfer object to hold the piece
   // information.
   m_transfer = new BlockTransfer();
-  m_transfer->create_dummy(m_peerChunks->peer_info(), piece);
+  Block::create_dummy(m_transfer, m_peerChunks->peer_info(), piece);
 
   return false;
 }
@@ -195,13 +195,20 @@ RequestList::transfer_dissimilar() {
     throw internal_error("RequestList::transfer_dissimilar() called but no transfer is in progress.");
 
   BlockTransfer* dummy = new BlockTransfer();
-  dummy->create_dummy(m_peerChunks->peer_info(), m_transfer->piece());
+  Block::create_dummy(dummy, m_peerChunks->peer_info(), m_transfer->piece());
   dummy->set_position(m_transfer->position());
 
   m_transfer->block()->transfer_dissimilar(m_transfer);
   m_transfer = dummy;
 }
 
+// void
+// RequestList::cancel_transfer(BlockTransfer* transfer) {
+// //   ReserveeList itr = std::find_if(m_canceled.begin(), m_canceled.end(), request_list_same_piece(piece));
+
+// //   ReserveeList itr = std::find_if(m_canceled.begin(), m_canceled.end(), request_list_same_piece(piece));
+// }
+
 struct equals_reservee : public std::binary_function<BlockTransfer*, uint32_t, bool> {
   bool operator () (BlockTransfer* r, uint32_t index) const {
     return r->is_valid() && index == r->index();
@@ -250,27 +257,6 @@ RequestList::cancel_range(ReserveeList::iterator end) {
   }
 }
 
-uint32_t
-RequestList::remove_invalid() {
-  uint32_t count = 0;
-  ReserveeList::iterator itr;
-
-  // Could be more efficient, but rarely do we find any.
-  while ((itr = std::find_if(m_queued.begin(), m_queued.end(),  std::not1(std::mem_fun(&BlockTransfer::is_valid)))) != m_queued.end()) {
-    count++;
-    Block::release(*itr);
-    m_queued.erase(itr);
-  }
-
-  // Don't count m_canceled that are invalid.
-  while ((itr = std::find_if(m_canceled.begin(), m_canceled.end(), std::not1(std::mem_fun(&BlockTransfer::is_valid)))) != m_canceled.end()) {
-    Block::release(*itr);
-    m_canceled.erase(itr);
-  }
-
-  return count;
-}
-
 uint32_t
 RequestList::calculate_pipe_size(uint32_t rate) {
   // Change into KB.
