@@ -41,6 +41,7 @@
 #include "net/listen.h"
 
 #include "connection_manager.h"
+#include "error.h"
 #include "exceptions.h"
 
 namespace torrent {
@@ -52,6 +53,7 @@ ConnectionManager::ConnectionManager() :
   m_priority(iptos_throughput),
   m_sendBufferSize(0),
   m_receiveBufferSize(0),
+  m_encryptionOptions(encryption_none),
 
   m_listen(new Listen) {
 
@@ -60,6 +62,9 @@ ConnectionManager::ConnectionManager() :
 
   m_localAddress = (new rak::socket_address())->c_sockaddr();
   rak::socket_address::cast_from(m_localAddress)->sa_inet()->clear();
+
+  m_proxyAddress = (new rak::socket_address())->c_sockaddr();
+  rak::socket_address::cast_from(m_proxyAddress)->sa_inet()->clear();
 }
 
 ConnectionManager::~ConnectionManager() {
@@ -84,6 +89,15 @@ ConnectionManager::set_receive_buffer_size(uint32_t s) {
   m_receiveBufferSize = s;
 }
 
+void
+ConnectionManager::set_encryption_options(uint32_t options) {
+#ifdef USE_OPENSSL
+  m_encryptionOptions = options;
+#else
+  throw input_error("Compiled without encryption support.");
+#endif
+}
+
 void
 ConnectionManager::set_bind_address(const sockaddr* sa) {
   const rak::socket_address* rsa = rak::socket_address::cast_from(sa);
@@ -104,6 +118,16 @@ ConnectionManager::set_local_address(const sockaddr* sa) {
   rak::socket_address::cast_from(m_localAddress)->copy(*rsa, rsa->length());
 }
 
+void
+ConnectionManager::set_proxy_address(const sockaddr* sa) {
+  const rak::socket_address* rsa = rak::socket_address::cast_from(sa);
+
+  if (rsa->family() != rak::socket_address::af_inet)
+    throw input_error("Tried to set a proxy address that is not an af_inet address.");
+
+  rak::socket_address::cast_from(m_proxyAddress)->copy(*rsa, rsa->length());
+}
+
 uint32_t
 ConnectionManager::filter(const sockaddr* sa) {
   if (m_slotFilter.empty())
