@@ -39,22 +39,20 @@
 #ifndef LIBTORRENT_CONNECTION_MANAGER_H
 #define LIBTORRENT_CONNECTION_MANAGER_H
 
-#include <inttypes.h>
 #include <sys/socket.h>
 #include <arpa/inet.h>
 #include <netinet/in.h>
 #include <netinet/in_systm.h>
 #include <netinet/ip.h>
+#include <sigc++/connection.h>
+#include <sigc++/signal.h>
 #include <sigc++/slot.h>
 
-class sockaddr;
+#include <torrent/common.h>
 
 namespace torrent {
 
-// Internal.
-class Listen;
-
-class ConnectionManager {
+class LIBTORRENT_EXPORT ConnectionManager {
 public:
   typedef uint32_t                              size_type;
   typedef uint16_t                              port_type;
@@ -74,6 +72,32 @@ public:
   static const priority_type iptos_mincost     = iptos_throughput;
 #endif
 
+  static const uint32_t encryption_none             = 0;
+  static const uint32_t encryption_allow_incoming   = (1 << 0);
+  static const uint32_t encryption_try_outgoing     = (1 << 1);
+  static const uint32_t encryption_require          = (1 << 2);
+  static const uint32_t encryption_require_RC4      = (1 << 3);
+  static const uint32_t encryption_enable_retry     = (1 << 4);
+  static const uint32_t encryption_prefer_plaintext = (1 << 5);
+
+  // Internal to libtorrent.
+  static const uint32_t encryption_use_proxy        = (1 << 6);
+
+  enum {
+    handshake_incoming           = 1,
+    handshake_outgoing           = 2,
+    handshake_outgoing_encrypted = 3,
+    handshake_outgoing_proxy     = 4,
+    handshake_success            = 5,
+    handshake_dropped            = 6,
+    handshake_failed             = 7,
+    handshake_retry_plaintext    = 8,
+    handshake_retry_encrypted    = 9
+  };
+
+  typedef sigc::slot4<void, const sockaddr*, int, int, const HashString*>   slot_handshake_type;
+  typedef sigc::signal4<void, const sockaddr*, int, int, const HashString*> signal_handshake_type;
+
   ConnectionManager();
   ~ConnectionManager();
   
@@ -101,6 +125,9 @@ public:
   uint32_t            receive_buffer_size() const             { return m_receiveBufferSize; }
   void                set_receive_buffer_size(uint32_t s);
 
+  uint32_t            encryption_options()                    { return m_encryptionOptions; }
+  void                set_encryption_options(uint32_t options); 
+
   // Propably going to have to make m_bindAddress a pointer to make it
   // safe.
   //
@@ -113,12 +140,18 @@ public:
   const sockaddr*     local_address() const                   { return m_localAddress; }
   void                set_local_address(const sockaddr* sa);
 
+  const sockaddr*     proxy_address() const                   { return m_proxyAddress; }
+  void                set_proxy_address(const sockaddr* sa);
+
   uint32_t            filter(const sockaddr* sa);
   void                set_filter(const slot_filter_type& s)   { m_slotFilter = s; }
 
   bool                listen_open(port_type begin, port_type end);
   void                listen_close();  
 
+  signal_handshake_type& signal_handshake_log()                          { return m_signalHandshakeLog; }
+  sigc::connection       set_signal_handshake_log(slot_handshake_type s) { return m_signalHandshakeLog.connect(s); }
+
   // Since trackers need our port number, it doesn't get cleared after
   // 'listen_close()'. The client may change the reported port number,
   // but do note that it gets overwritten after 'listen_open(...)'.
@@ -138,14 +171,17 @@ private:
   priority_type       m_priority;
   uint32_t            m_sendBufferSize;
   uint32_t            m_receiveBufferSize;
+  int                 m_encryptionOptions;
 
   sockaddr*           m_bindAddress;
   sockaddr*           m_localAddress;
+  sockaddr*           m_proxyAddress;
 
   Listen*             m_listen;
   port_type           m_listenPort;
 
-  slot_filter_type    m_slotFilter;
+  slot_filter_type      m_slotFilter;
+  signal_handshake_type m_signalHandshakeLog;
 };
 
 }
