@@ -40,6 +40,8 @@
 #include <sigc++/bind.h>
 #include <sigc++/hide.h>
 
+#include "data/block.h"
+#include "data/block_list.h"
 #include "data/chunk_list.h"
 #include "data/hash_queue.h"
 #include "data/hash_torrent.h"
@@ -51,15 +53,12 @@
 #include "protocol/peer_connection_base.h"
 #include "protocol/peer_factory.h"
 #include "download/download_info.h"
+#include "peer/peer_info.h"
 #include "tracker/tracker_manager.h"
 
 #include "exceptions.h"
-#include "block.h"
-#include "block_list.h"
 #include "download.h"
-#include "file_list.h"
 #include "object.h"
-#include "peer_info.h"
 #include "tracker_list.h"
 
 namespace torrent {
@@ -80,10 +79,10 @@ Download::close() {
 void
 Download::start() {
   if (!m_ptr->hash_checker()->is_checked())
-    throw client_error("Tried to start an unchecked download");
+    throw internal_error("Tried to start an unchecked download.");
 
   if (!m_ptr->info()->is_open())
-    throw client_error("Tried to start a closed download");
+    throw internal_error("Tried to start a closed download.");
 
   if (m_ptr->info()->is_active())
     return;
@@ -123,24 +122,25 @@ Download::hash_check(bool tryQuick) {
     return m_ptr->hash_checker()->start(tryQuick);
 
   if (!m_ptr->info()->is_open() || m_ptr->info()->is_active())
-    throw client_error("Download::hash_check(...) called on a closed or active download.");
+    throw internal_error("Download::hash_check(...) called on a closed or active download.");
 
   if (m_ptr->hash_checker()->is_checked())
-    throw client_error("Download::hash_check(...) called but already hash checked.");
+    throw internal_error("Download::hash_check(...) called but already hash checked.");
 
   // The bitfield still hasn't been allocated, so no resume data was
   // given. 
+  Bitfield* bitfield = m_ptr->main()->file_list()->mutable_bitfield();
 
-  if (m_ptr->main()->content()->bitfield()->empty()) {
-    m_ptr->main()->content()->bitfield()->allocate();
-    m_ptr->main()->content()->bitfield()->unset_all();
+  if (bitfield->empty()) {
+    bitfield->allocate();
+    bitfield->unset_all();
 
-    m_ptr->hash_checker()->ranges().insert(0, m_ptr->main()->content()->chunk_total());
+    m_ptr->hash_checker()->ranges().insert(0, m_ptr->main()->file_list()->size_chunks());
 
     // Make sure other book-keeping is cleared, like file progress.
 
   } else {
-    m_ptr->main()->content()->update_done();
+    m_ptr->main()->file_list()->update_completed();
   }
 
   return m_ptr->hash_checker()->start(tryQuick);
@@ -152,13 +152,10 @@ Download::hash_stop() {
   if (!m_ptr->hash_checker()->is_checking())
     return;
 
-  // Stop the hashing first as we need to make sure all chunks are
-  // released when DownloadMain::close() is called.
-  m_ptr->hash_checker()->clear();
+  m_ptr->hash_checker()->ranges().erase(0, m_ptr->hash_checker()->position());
+  m_ptr->hash_queue()->remove(m_ptr);
 
-  // Clear after m_hash to ensure that the empty hash done signal does
-  // not get passed to HashTorrent.
-  m_ptr->hash_checker()->get_queue()->remove(m_ptr);
+  m_ptr->hash_checker()->clear();
 }
 
 bool
@@ -189,19 +186,18 @@ Download::name() const {
   return m_ptr->info()->name();
 }
 
-const std::string&
+const HashString&
 Download::info_hash() const {
-  if (m_ptr == NULL)
-    throw internal_error("Download::info_hash() m_ptr == NULL.");
-
   return m_ptr->info()->hash();
 }
 
-const std::string&
-Download::local_id() const {
-  if (m_ptr == NULL)
-    throw internal_error("Download::local_id() m_ptr == NULL.");
+const HashString&
+Download::info_hash_obfuscated() const {
+  return m_ptr->info()->hash_obfuscated();
+}
 
+const HashString&
+Download::local_id() const {
   return m_ptr->info()->local_id();
 }
 
@@ -223,9 +219,9 @@ Download::bencode() const {
   return m_ptr->bencode();
 }
 
-FileList
+FileList*
 Download::file_list() const {
-  return FileList(m_ptr->main()->content()->entry_list());
+  return m_ptr->main()->file_list();
 }
 
 TrackerList
@@ -267,6 +263,16 @@ Download::up_rate() const {
   return m_ptr->info()->up_rate();
 }
 
+Rate*
+Download::skip_rate() {
+  return m_ptr->info()->skip_rate();
+}
+
+const Rate*
+Download::skip_rate() const {
+  return m_ptr->info()->skip_rate();
+}
+
 uint64_t
 Download::bytes_done() const {
   uint64_t a = 0;
@@ -278,32 +284,7 @@ Download::bytes_done() const {
       if (itr2->is_finished())
         a += itr2->piece().length();
   
-  return a + m_ptr->main()->content()->bytes_completed();
-}
-
-uint64_t
-Download::bytes_total() const {
-  return m_ptr->main()->content()->entry_list()->bytes_size();
-}
-
-uint64_t
-Download::free_diskspace() const {
-  return m_ptr->main()->content()->entry_list()->free_diskspace();
-}
-
-uint32_t
-Download::chunks_size() const {
-  return m_ptr->main()->content()->chunk_size();
-}
-
-uint32_t
-Download::chunks_done() const {
-  return m_ptr->main()->content()->chunks_completed();
-}
-
-uint32_t 
-Download::chunks_total() const {
-  return m_ptr->main()->content()->chunk_total();
+  return a + m_ptr->main()->file_list()->completed_bytes();
 }
 
 uint32_t 
@@ -321,7 +302,7 @@ Download::set_chunks_done(uint32_t chunks) {
   if (m_ptr->info()->is_open())
     throw input_error("Download::set_chunks_done(...) Download is open.");
 
-  m_ptr->main()->content()->bitfield()->set_size_set(chunks);
+  m_ptr->main()->file_list()->mutable_bitfield()->set_size_set(chunks);
 }
 
 void
@@ -329,7 +310,7 @@ Download::set_bitfield(bool allSet) {
   if (m_ptr->hash_checker()->is_checked() || m_ptr->hash_checker()->is_checking())
     throw input_error("Download::set_bitfield(...) Download in invalid state.");
 
-  Bitfield* bitfield = m_ptr->main()->content()->bitfield();
+  Bitfield* bitfield = m_ptr->main()->file_list()->mutable_bitfield();
 
   bitfield->allocate();
 
@@ -346,10 +327,10 @@ Download::set_bitfield(uint8_t* first, uint8_t* last) {
   if (m_ptr->hash_checker()->is_checked() || m_ptr->hash_checker()->is_checking())
     throw input_error("Download::set_bitfield(...) Download in invalid state.");
 
-  if (std::distance(first, last) != (ptrdiff_t)m_ptr->main()->content()->bitfield()->size_bytes())
+  if (std::distance(first, last) != (ptrdiff_t)m_ptr->main()->file_list()->bitfield()->size_bytes())
     throw input_error("Download::set_bitfield(...) Invalid length.");
 
-  Bitfield* bitfield = m_ptr->main()->content()->bitfield();
+  Bitfield* bitfield = m_ptr->main()->file_list()->mutable_bitfield();
 
   bitfield->allocate();
   std::memcpy(bitfield->begin(), first, bitfield->size_bytes());
@@ -360,18 +341,13 @@ Download::set_bitfield(uint8_t* first, uint8_t* last) {
 
 void
 Download::clear_range(uint32_t first, uint32_t last) {
-  if (m_ptr->hash_checker()->is_checked() || m_ptr->hash_checker()->is_checking() || m_ptr->main()->content()->bitfield()->empty())
+  if (m_ptr->hash_checker()->is_checked() || m_ptr->hash_checker()->is_checking() || m_ptr->main()->file_list()->bitfield()->empty())
     throw input_error("Download::clear_range(...) Download in invalid state.");
 
   m_ptr->hash_checker()->ranges().insert(first, last);
-  m_ptr->main()->content()->bitfield()->unset_range(first, last);
+  m_ptr->main()->file_list()->mutable_bitfield()->unset_range(first, last);
 }
  
-const Bitfield*
-Download::bitfield() const {
-  return m_ptr->main()->content()->bitfield();
-}
-
 void
 Download::sync_chunks() {
   m_ptr->main()->chunk_list()->sync_chunks(ChunkList::sync_all | ChunkList::sync_force);
@@ -489,7 +465,7 @@ Peer
 Download::peer_find(const std::string& id) {
   ConnectionList::iterator itr =
     std::find_if(m_ptr->main()->connection_list()->begin(), m_ptr->main()->connection_list()->end(),
-                 rak::equal(id, rak::on(std::mem_fun(&PeerConnectionBase::c_peer_info), std::mem_fun(&PeerInfo::id))));
+                 rak::equal(*HashString::cast_from(id), rak::on(std::mem_fun(&PeerConnectionBase::c_peer_info), std::mem_fun(&PeerInfo::id))));
 
   return itr != m_ptr->main()->connection_list()->end() ? *itr : NULL;
 }
