@@ -37,33 +37,23 @@
 #ifndef LIBTORRENT_DOWNLOAD_H
 #define LIBTORRENT_DOWNLOAD_H
 
-#include <torrent/peer.h>
-
 #include <list>
 #include <vector>
 #include <string>
-#include <inttypes.h>
 #include <sigc++/slot.h>
 #include <sigc++/connection.h>
 
+#include <torrent/common.h>
+#include <torrent/peer/peer.h>
+
 namespace torrent {
 
 typedef std::list<Peer> PList;
 
-class Bitfield;
-class DownloadWrapper;
-class FileList;
-class Rate;
-class Object;
-class Peer;
-class PeerList;
-class TrackerList;
-class TransferList;
-
 // Download is safe to copy and destory as it is just a pointer to an
 // internal class.
 
-class Download {
+class LIBTORRENT_EXPORT Download {
 public:
   static const uint32_t numwanted_diabled = ~uint32_t();
 
@@ -97,8 +87,10 @@ public:
 
   // Returns "" if the object is not valid.
   const std::string&  name() const;
-  const std::string&  info_hash() const;
-  const std::string&  local_id() const;
+
+  const HashString&   info_hash() const;
+  const HashString&   info_hash_obfuscated() const;
+  const HashString&   local_id() const;
 
   // Unix epoche, 0 == unknown.
   uint32_t            creation_date() const;
@@ -106,9 +98,9 @@ public:
   Object*             bencode();
   const Object*       bencode() const;
 
-  FileList            file_list() const;
   TrackerList         tracker_list() const;
 
+  FileList*           file_list() const;
   PeerList*           peer_list();
   const PeerList*     peer_list() const;
   const TransferList* transfer_list() const;
@@ -119,16 +111,12 @@ public:
   Rate*               up_rate();
   const Rate*         up_rate() const;
 
+  Rate*               skip_rate();
+  const Rate*         skip_rate() const;
+
   // Bytes completed.
   uint64_t            bytes_done() const;
-  // Size of the torrent.
-  uint64_t            bytes_total() const;
 
-  uint64_t            free_diskspace() const;
-
-  uint32_t            chunks_size() const;
-  uint32_t            chunks_done() const;
-  uint32_t            chunks_total() const;
   uint32_t            chunks_hashed() const;
 
   const uint8_t*      chunks_seen() const;
@@ -145,8 +133,6 @@ public:
   void                set_bitfield(uint8_t* first, uint8_t* last);
   void                clear_range(uint32_t first, uint32_t last);
 
-  const Bitfield*     bitfield() const;
-
   // Temporary hack for syncing chunks to disk before hash resume is
   // saved.
   void                sync_chunks();
