@@ -41,15 +41,16 @@
 
 // For SocketAddressCompact.
 #include "download/download_info.h"
+#include "peer/peer_info.h"
+#include "peer/peer_list.h"
+
+#include "data/file.h"
+#include "data/file_list.h"
 
 #include "common.h"
 #include "bitfield.h"
 #include "download.h"
-#include "file.h"
-#include "file_list.h"
 #include "object.h"
-#include "peer_info.h"
-#include "peer_list.h"
 #include "tracker.h"
 #include "tracker_list.h"
 
@@ -66,13 +67,13 @@ resume_load_progress(Download download, const Object& object) {
 
   const Object::list_type& files = object.get_key_list("files");
 
-  if (files.size() != download.file_list().size())
+  if (files.size() != download.file_list()->size_files())
     return;
 
   if (object.has_key_string("bitfield")) {
     const Object::string_type& bitfield = object.get_key_string("bitfield");
 
-    if (bitfield.size() != download.bitfield()->size_bytes())
+    if (bitfield.size() != download.file_list()->bitfield()->size_bytes())
       return;
 
     download.set_bitfield((uint8_t*)bitfield.c_str(), (uint8_t*)(bitfield.c_str() + bitfield.size()));
@@ -80,7 +81,7 @@ resume_load_progress(Download download, const Object& object) {
   } else if (object.has_key_value("bitfield")) {
     Object::value_type chunksDone = object.get_key_value("bitfield");
 
-    if (chunksDone == download.bitfield()->size_bits())
+    if (chunksDone == download.file_list()->bitfield()->size_bits())
       download.set_bitfield(true);
     else if (chunksDone == 0)
       download.set_bitfield(false);
@@ -94,18 +95,17 @@ resume_load_progress(Download download, const Object& object) {
   Object::list_type::const_iterator filesItr  = files.begin();
   Object::list_type::const_iterator filesLast = files.end();
 
-  FileList fileList = download.file_list();
+  FileList* fileList = download.file_list();
 
-  for (unsigned int index = 0; index < fileList.size(); ++index, ++filesItr) {
+  for (FileList::iterator listItr = fileList->begin(), listLast = fileList->end(); listItr != listLast; ++listItr, ++filesItr) {
     rak::file_stat fs;
-    File file = fileList.get(index);
 
     // Check that the size and modified stamp matches. If not, then
     // clear the resume data for that range.
 
-    if (!fs.update(fileList.root_dir() + file.path_str()) || fs.size() != (off_t)file.size_bytes() ||
+    if (!fs.update(fileList->root_dir() + (*listItr)->path()->as_string()) || (uint64_t)fs.size() != (*listItr)->size_bytes() ||
         !filesItr->has_key_value("mtime") || filesItr->get_key_value("mtime") != fs.modified_time())
-      download.clear_range(file.chunk_begin(), file.chunk_end());
+      download.clear_range((*listItr)->range().first, (*listItr)->range().second);
   }
 }
 
@@ -122,12 +122,12 @@ resume_save_progress(Download download, Object& object, bool onlyCompleted) {
   if (!download.is_hash_checked())
     return;
 
-  const Bitfield* bitfield = download.bitfield();
+  const Bitfield* bitfield = download.file_list()->bitfield();
 
   if (bitfield->is_all_set() || bitfield->is_all_unset())
     object.insert_key("bitfield", bitfield->size_set());
   else
-    object.insert_key("bitfield", std::string((char*)download.bitfield()->begin(), download.bitfield()->size_bytes()));
+    object.insert_key("bitfield", std::string((char*)bitfield->begin(), bitfield->size_bytes()));
   
   Object::list_type& files = object.has_key_list("files")
     ? object.get_key_list("files")
@@ -135,19 +135,18 @@ resume_save_progress(Download download, Object& object, bool onlyCompleted) {
 
   Object::list_type::iterator filesItr = files.begin();
 
-  FileList fileList = download.file_list();
+  FileList* fileList = download.file_list();
 
-  for (unsigned int index = 0; index < fileList.size(); ++index, ++filesItr) {
+  for (FileList::iterator listItr = fileList->begin(), listLast = fileList->end(); listItr != listLast; ++listItr, ++filesItr) {
     if (filesItr == files.end())
       filesItr = files.insert(filesItr, Object(Object::TYPE_MAP));
     else if (!filesItr->is_map())
       *filesItr = Object(Object::TYPE_MAP);
 
     rak::file_stat fs;
-    File file = fileList.get(index);
 
-    if (!fs.update(fileList.root_dir() + file.path_str()) ||
-        (onlyCompleted && file.completed_chunks() != file.size_chunks())) {
+    if (!fs.update(fileList->root_dir() + (*listItr)->path()->as_string()) ||
+        (onlyCompleted && (*listItr)->completed_chunks() != (*listItr)->size_chunks())) {
       filesItr->erase_key("mtime");
       continue;
     }
@@ -171,16 +170,16 @@ resume_load_file_priorities(Download download, const Object& object) {
   Object::list_type::const_iterator filesItr  = files.begin();
   Object::list_type::const_iterator filesLast = files.end();
 
-  FileList fileList = download.file_list();
+  FileList* fileList = download.file_list();
 
-  for (unsigned int index = 0; index < fileList.size(); ++index, ++filesItr) {
+  for (FileList::iterator listItr = fileList->begin(), listLast = fileList->end(); listItr != listLast; ++listItr, ++filesItr) {
     if (filesItr == filesLast)
       return;
 
     // Update the priority from the fast resume data.
     if (filesItr->has_key_value("priority") &&
         filesItr->get_key_value("priority") >= 0 && filesItr->get_key_value("priority") <= PRIORITY_HIGH)
-      fileList.get(index).set_priority((priority_t)filesItr->get_key_value("priority"));
+      (*listItr)->set_priority((priority_t)filesItr->get_key_value("priority"));
   }
 }
 
@@ -192,15 +191,15 @@ resume_save_file_priorities(Download download, Object& object) {
 
   Object::list_type::iterator filesItr = files.begin();
 
-  FileList fileList = download.file_list();
+  FileList* fileList = download.file_list();
 
-  for (unsigned int index = 0; index < fileList.size(); ++index, ++filesItr) {
+  for (FileList::iterator listItr = fileList->begin(), listLast = fileList->end(); listItr != listLast; ++listItr, ++filesItr) {
     if (filesItr == files.end())
       filesItr = files.insert(filesItr, Object(Object::TYPE_MAP));
     else if (!filesItr->is_map())
       *filesItr = Object(Object::TYPE_MAP);
 
-    filesItr->insert_key("priority", (int64_t)fileList.get(index).priority());
+    filesItr->insert_key("priority", (int64_t)(*listItr)->priority());
   }
 }
 
