@@ -94,10 +94,10 @@ calculate_reserved(uint32_t openMax) {
 void
 initialize(Poll* poll) {
   if (manager != NULL)
-    throw client_error("torrent::initialize(...) called but the library has already been initialized");
+    throw internal_error("torrent::initialize(...) called but the library has already been initialized");
 
   if (poll->open_max() < 64)
-    throw client_error("Could not initialize libtorrent, Poll::open_max() < 64.");
+    throw internal_error("Could not initialize libtorrent, Poll::open_max() < 64.");
 
   cachedTime = rak::timer::current();
 
@@ -115,7 +115,7 @@ initialize(Poll* poll) {
 void
 cleanup() {
   if (manager == NULL)
-    throw client_error("torrent::cleanup() called but the library is not initialized.");
+    throw internal_error("torrent::cleanup() called but the library is not initialized.");
 
   delete manager;
   manager = NULL;
@@ -155,6 +155,11 @@ chunk_manager() {
   return manager->chunk_manager();
 }
 
+ClientList*
+client_list() {
+  return manager->client_list();
+}
+
 ConnectionManager*
 connection_manager() {
   return manager->connection_manager();
@@ -291,24 +296,6 @@ set_max_open_files(uint32_t size) {
   manager->file_manager()->set_max_size(size);
 }
 
-uint32_t
-open_sockets() {
-  return manager->connection_manager()->size();
-}
-
-uint32_t
-max_open_sockets() {
-  return manager->connection_manager()->max_size();
-}
-
-void
-set_max_open_sockets(uint32_t size) {
-  if (size < 4 || size > (1 << 16))
-    throw input_error("Max open sockets must be between 4 and 2^16.");
-
-  manager->connection_manager()->set_max_size(size);
-}
-
 EncodingList*
 encoding_list() {
   return manager->encoding_list();
@@ -329,6 +316,7 @@ download_add(Object* object) {
   if (manager->download_manager()->find(infoHash) != manager->download_manager()->end())
     throw input_error("Info hash already used by another torrent.");
 
+  download->set_hash_queue(manager->hash_queue());
   download->initialize(infoHash, PEER_NAME + rak::generate_random<std::string>(20 - std::string(PEER_NAME).size()));
 
   // Default PeerConnection factory functions.
@@ -367,7 +355,7 @@ download_priority(Download d) {
   ResourceManager::iterator itr = manager->resource_manager()->find(d.ptr()->main());
 
   if (itr == manager->resource_manager()->end())
-    throw client_error("torrent::download_priority(...) could not find the download in the resource manager.");
+    throw internal_error("torrent::download_priority(...) could not find the download in the resource manager.");
 
   return itr->first;
 }
@@ -377,10 +365,10 @@ download_set_priority(Download d, uint32_t pri) {
   ResourceManager::iterator itr = manager->resource_manager()->find(d.ptr()->main());
 
   if (itr == manager->resource_manager()->end())
-    throw client_error("torrent::download_set_priority(...) could not find the download in the resource manager.");
+    throw internal_error("torrent::download_set_priority(...) could not find the download in the resource manager.");
 
   if (pri > 1024)
-    throw client_error("torrent::download_set_priority(...) received an invalid priority.");
+    throw internal_error("torrent::download_set_priority(...) received an invalid priority.");
 
   manager->resource_manager()->set_priority(itr, pri);
 }
